/*
 * xkeymacs substitute for X11/Linux
 * Copyright 2011 (C) Takeshi Yashiro <yashiromann@nifty.com>
 *
 * conv.h: keyboard input converter (abstract)
 */

#ifndef _DEFINE_H_X11KEYMACS_EMACS_
#define _DEFINE_H_X11KEYMACS_EMACS_

#include <vector>
#include <string>
#include <linux/input.h>
#include <boost/regex.hpp>
#include "conv.h"

class EmacsConverter : public Converter
{
public:
    EmacsConverter(const std::vector<std::string>& excludeList, bool debug = false);
    virtual ~EmacsConverter();

protected:
    virtual bool handleInput(struct input_event* input);

    bool handleKeyInput(struct input_event* input);
    bool handleMetaKeyInput(struct input_event* input, int bit);
    bool handleKeyDefault(struct input_event* input);
    void typeKey(__u16 code, int metaKeys);

    void addExcludePattern(const char* pattern);
    bool exclude() const;

private:
    int m_metaKeyFlags;

    bool m_state_ctrlx;
    bool m_state_ctrlspc;
    bool m_state_ctrlk;
    bool m_state_pasteNewline;
    typedef std::vector<boost::regex*> ExcludeList;
    ExcludeList m_excludePatterns;

    bool m_debug;
};

#endif /* _DEFINE_H_X11KEYMACS_EMACS_ */
