/*
 * xkeymacs substitute for X11/Linux
 * Copyright 2011 (C) Takeshi Yashiro <yashiromann@nifty.com>
 *
 * window.cpp: window system
 */

#include <stdio.h>
#include <sched.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include "window.h"

WindowSystem* WindowSystem::m_instance = NULL;

WindowSystem* WindowSystem::getInstance()
{
    if (m_instance == NULL)
        m_instance = new WindowSystem();
    return m_instance;
}

WindowSystem::WindowSystem()
    : m_xerr(false)
{
    /* setlocale somehow needed */
    setlocale(LC_ALL, "");
    m_display = XOpenDisplay(NULL);
    XSetErrorHandler(errorHandler);
}

WindowSystem::~WindowSystem()
{
    XCloseDisplay(m_display);
}

int WindowSystem::errorHandler(Display* display, XErrorEvent* err)
{
    fprintf(stderr, "WARNING: X11 error (window destroyed?)\n");
    WindowSystem::getInstance()->m_xerr = true;

    return 1;
}

std::string WindowSystem::getFocusedWindowTitle()
{
    XTextProperty wname;
    std::string retval;
    Window window;
    int revert_to;
    Window root, parent;
    Window* children;
    unsigned int nc;
    XWMHints *wmhints = NULL;
    int result;

 again:
    XGetInputFocus(m_display, &window, &revert_to);
    Window w = window;

    for (;;) {
        m_xerr = false;

        /* It seems that, when XWMHints are available, we can get window titles
           which are same to those fetched in window manager (KDE) */
        wmhints = XGetWMHints(m_display, w);
        if (m_xerr) {
            sched_yield();
            goto again;
        }
        if (wmhints != NULL) {
            XFree(wmhints);
            break;
        }

        /* Seek for parent window */
        result = XQueryTree(m_display, w, &root, &parent, &children, &nc);
        if (m_xerr) {
            sched_yield();
            goto again;
        }
        if (result) {
            if (children)
                XFree(children);
            if (w == root || parent == root)
                break;
            w = parent;
        }
        else break;
    }

    /* Get the window name */
    result = XGetWMName(m_display, w, &wname);
    if (result && wname.nitems > 0) {
        int count = 0, i, ret;
        char **list = NULL;
        ret = XmbTextPropertyToTextList(m_display, &wname, &list, &count);
        if (m_xerr) {
            sched_yield();
            goto again;
        }
        if ((ret == Success || ret > 0) && list != NULL) {
            for (i=0; i<count; i++)
                retval.append(list[i]);
            XFreeStringList(list);
        } else if (wname.value != NULL) {
            retval.append((char*)wname.value);
        }
    }

    return retval;
}
